#include <pds/pdsdatafunc.h>
#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QMessageBox>
#include <QFileDialog>

//#include <QDebug>
//#include <QUrl>
//#include <QDesktopServices>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    setWindowTitle(QString(APP_TARGET)+" "+QString(APP_VERSION));
    ui->mainToolBar->setToolButtonStyle( Qt::ToolButtonTextUnderIcon );
    this->SORTDATATYPE=SORT_LEX;
}

int MainWindow::set_nfiles(void)
{
    int N=ui->tableWidget->rowCount();
    ui->label_nfiles->setText(QString::number(N));
    return N;
}
void MainWindow::search_files(void)
{
    QString rootdir=ui->lineEdit_rootdir->text();
    QString filter=ui->lineEdit_filter->text();

    QByteArray ba_rootdir = rootdir.toLocal8Bit();
    QByteArray ba_filter  = filter.toLocal8Bit();

    PdsCellString * filepath=pds_get_arquives_new(ba_rootdir.data(),TYPE_FILE,ba_filter.data(),".*");
    int N=pds_cell_string_get_ncell (filepath);


    if(this->SORTDATATYPE==SORT_LEX)
        pds_cell_string_sort_by_lexicographic_order(filepath);
    else if(this->SORTDATATYPE==SORT_LENGTH)
        pds_cell_string_sort_by_length(filepath);
    else
        pds_cell_string_sort_by_lexicographic_order(filepath);

    char *tmp=NULL;

    //ui->tableWidget->horizontalHeader()->setStretchLastSection(true);
    ui->tableWidget->clearContents();
    ui->tableWidget->setRowCount(0);

    //tabla con N lineas y una columna
    ui->tableWidget->setRowCount(N);
    ui->tableWidget->setColumnCount(1);

    QStringList titulo;
    titulo<<"filepath";
    ui->tableWidget->setHorizontalHeaderLabels(titulo);
    ui->tableWidget->horizontalHeader()->setVisible(true);

    for(int i=0;i<N;i++)
    {
        pds_cell_string_read(filepath,i,&tmp);
        QTableWidgetItem *item = new QTableWidgetItem(QString::fromLocal8Bit(tmp));
        item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
        ui->tableWidget->setItem(i, 0, item);
    }

    ui->tableWidget->setVisible(false);
    ui->tableWidget->resizeColumnsToContents();
    ui->tableWidget->setVisible(true);

    pds_cell_string_free(filepath);

    set_nfiles();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_pushButton_search_clicked()
{
    search_files();
}

void MainWindow::on_pushButton_deleterow_clicked()
{

    for(int i=(ui->tableWidget->rowCount()-1); i>=0 ; i--)
    {
        if(ui->tableWidget->item(i,0)->isSelected()==true)
            ui->tableWidget->removeRow(i);
    }
    set_nfiles();
}

void MainWindow::on_pushButton_rootdir_clicked()
{

    QString  new_rootdir =  QFileDialog::getExistingDirectory(0, ("Select root directory"), ui->lineEdit_rootdir->text());//QDir::currentPath());
    ui->lineEdit_rootdir->setText(new_rootdir);
}

void MainWindow::on_pushButton_outfile_clicked()
{
    QString  origem;

    if(ui->lineEdit_outfile->text().count()!=0)
        origem=QFileInfo(ui->lineEdit_outfile->text()).path()+QDir::separator()+"listfilesdat.listfiles";
    else
        origem=QDir::homePath()+QDir::separator()+"listfilesdat.listfiles";

    QString  fileName = QFileDialog::getSaveFileName(this,
                                                      tr("Select or define an output filename"),
                                                      origem,
                                                      tr("Output listfiles file (*.listfiles);;Output data file (*.dat);;All files (*)"));
    ui->lineEdit_outfile->setText(fileName);
}

void MainWindow::set_parameter_progpath(const char *ppath)
{
    this->progpath=ppath;
}

void MainWindow::set_parameter_progdir(const char *pdir)
{
    this->progdir=pdir;
}


