#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QMessageBox>
#include <QDir>
#include <QImageReader>
#include <QFileDialog>
#include <QGraphicsScene>

#include <math.h>
#include <QStringList>
#include <iostream>

#include <QDebug>
#include <QUrl>
#include <QDesktopServices>

#include <QColorDialog>

#include <pds/pdsdatafunc.h>
#include <pdsmm/regionrect.h>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    this->scene=NULL;
    this->NCLICKS=0;
    this->ONE_BY_ONE_CLICKED=false;
    this->GRID_C1_CLICKED=false;
    this->GRID_C2_CLICKED=false;

    this->color_point =QColor(255,  0,0);
    this->color_corner=QColor(  0,255,0);

    this->pen_point =QPen(this->color_point);
    this->pen_corner=QPen(this->color_corner);
    this->pen_corner.setWidth(5);



    load_imagefile("");
    clear_regions();

    ui->mainToolBar->setToolButtonStyle( Qt::ToolButtonTextUnderIcon );

    connect(ui->graphicsView,SIGNAL(sendMousePosition(QPoint &)),this,SLOT(my_mouse_event(QPoint&)) );

    setWindowTitle(QString(APP_TARGET)+" "+QString(APP_VERSION));

    QIcon *icop = new QIcon();
    QPixmap pmp=QPixmap(32,32); pmp.fill(this->color_point);
    icop->addPixmap(pmp,QIcon::Normal,QIcon::On);
    ui->pushButton_point_color->setIcon(*icop);


    QIcon *icoc = new QIcon();
    QPixmap pmc=QPixmap(32,32); pmc.fill(this->color_corner);
    icoc->addPixmap(pmc,QIcon::Normal,QIcon::On);
    ui->pushButton_corner_color->setIcon(*icoc);

}


void MainWindow::calcular_el_valor_de_nlin_ncol(int *nlin=NULL,int *ncol=NULL)
{
    QPoint p1,p2;
    int NLIN,NCOL;

    int NV=ui->spinBox_grid_nvregions->value();
    int NH=ui->spinBox_grid_nhregions->value();

    p1.setX(ui->spinBox_grid_c1_column->value());
    p1.setY(ui->spinBox_grid_c1_line->value());

    p2.setX(ui->spinBox_grid_c2_column->value());
    p2.setY(ui->spinBox_grid_c2_line->value());

    NLIN=abs((1+p2.y()-p1.y())/NV);
    NCOL=abs((1+p2.x()-p1.x())/NH);

    ui->label_grid_nlin->setText(QString::number(NLIN));
    ui->label_grid_ncol->setText(QString::number(NCOL));

    if(nlin!=NULL)  *nlin=NLIN;
    if(ncol!=NULL)  *ncol=NCOL;
}


MainWindow::~MainWindow()
{
    delete ui;
}



//////////////////////////////////////////////////////////////////////
/// TITLE BUTTONS
//////////////////////////////////////////////////////////////////////


void MainWindow::on_pushButton_imagefile_clicked()
{
    QString  origem;

    if(ui->lineEdit_imagefile->text().count()!=0)
        origem=ui->lineEdit_imagefile->text();
    else
        origem=QDir::homePath();//QDir::currentPath();

    QString  imagefile = QFileDialog::getOpenFileName(this,
                                                          tr("Open Image File"),
                                                          origem,
                                                          tr("Images (*.png *.bmp *.jpg)"));

    if(load_imagefile(imagefile)==true)
        ui->lineEdit_imagefile->setText(imagefile);
    else
        ui->lineEdit_imagefile->setText("");

}

void MainWindow::on_pushButton_outfile_clicked()
{
    QString  origem;

    if(ui->lineEdit_outfile->text().count()!=0)
        origem=QFileInfo(ui->lineEdit_outfile->text()).path()+QDir::separator()+"listregionsdat.listregions";
    else
        origem=QDir::homePath()+QDir::separator()+"listregionsdat.listregions";

    QString  fileName = QFileDialog::getSaveFileName(this,
                                                     tr("Select or define an output filename"),
                                                     origem,
                                                     tr("Output listregions file (*.listregions);;Output data file (*.dat);;All files (*.*)"));
    ui->lineEdit_outfile->setText(fileName  );
}


void MainWindow::on_pushButton_select_one_by_one_clicked()
{
    this->NCLICKS=0;
    this->ONE_BY_ONE_CLICKED=true;
    ui->statusBar->showMessage(tr("Select the central point"), 3000);
}


void MainWindow::on_pushButton_select_grid_clicked()
{
    QPoint p1,p2;
    int nlin;
    int ncol;

    int NV=ui->spinBox_grid_nvregions->value();
    int NH=ui->spinBox_grid_nhregions->value();

    p1.setX(ui->spinBox_grid_c1_column->value());
    p1.setY(ui->spinBox_grid_c1_line->value());

    p2.setX(ui->spinBox_grid_c2_column->value());
    p2.setY(ui->spinBox_grid_c2_line->value());

    if( (p1.x()<0)||(p1.y()<0)||(p1.x()>=IMAGEW)||(p1.y()>=IMAGEH) )
    {
        QMessageBox Msgbox;
        Msgbox.setText(tr("The corner 1 is outside the image."));
        Msgbox.exec();
    }
    else if( (p2.x()<0)||(p2.y()<0)||(p2.x()>=IMAGEW)||(p2.y()>=IMAGEH) )
    {
        QMessageBox Msgbox;
        Msgbox.setText(tr("The corner 2 is outside the image."));
        Msgbox.exec();
    }
    else
    {
        calcular_el_valor_de_nlin_ncol(&nlin,&ncol);
        QPoint p0( std::min(p2.x(),p1.x()),std::min(p2.y(),p1.y()) );

        QPoint p;
        for(int i=0;i<NV;i++)
        for(int j=0;j<NH;j++)
        {
            p.setY(p0.y()+i*nlin);
            p.setX(p0.x()+j*ncol);

            Pds::RegionRect RR(p.y(),p.x(),nlin,ncol);
            pointl_p0.append(RR);
            ui->label_nregions->setText(QString::number(pointl_p0.size()));

            plot_regionrect(RR,this->pen_point);
        }


        plot_point(p1,this->pen_corner);

        plot_point(p2,this->pen_corner);

        ui->statusBar->showMessage(tr("Regions selected"), 3000);
    }
}

void MainWindow::on_pushButton_grid_c1_clicked()
{
    this->NCLICKS=0;
    this->GRID_C1_CLICKED=true;
    ui->statusBar->showMessage(tr("Select a corner point 1"), 3000);

}

void MainWindow::on_pushButton_grid_c2_clicked()
{
    this->NCLICKS=0;
    this->GRID_C2_CLICKED=true;
    ui->statusBar->showMessage(tr("Select a corner point 2"), 3000);


}


void MainWindow::on_pushButton_saveexit_clicked()
{
    if(save_in_outfile()==true)
    QApplication::quit();
}

void MainWindow::on_pushButton_point_color_clicked()
{
    QColor color = QColorDialog::getColor(this->color_point, this );
    if( color.isValid() )
    {
        this->color_point=color;
        this->pen_point =QPen(this->color_point);


        QIcon *icop = new QIcon();
        QPixmap pmp=QPixmap(32,32); pmp.fill(this->color_point);
        icop->addPixmap(pmp,QIcon::Normal,QIcon::On);
        ui->pushButton_point_color->setIcon(*icop);
    }



}

void MainWindow::on_pushButton_corner_color_clicked()
{
    QColor color = QColorDialog::getColor(this->color_corner, this );
    if( color.isValid() )
    {
        this->color_corner=color;
        this->pen_corner =QPen(this->color_corner);
        this->pen_corner.setWidth(5);

        QIcon *icop = new QIcon();
        QPixmap pmp=QPixmap(32,32); pmp.fill(this->color_corner);
        icop->addPixmap(pmp,QIcon::Normal,QIcon::On);
        ui->pushButton_corner_color->setIcon(*icop);
    }
}


void MainWindow::on_spinBox_grid_nvregions_valueChanged(int arg1)
{
    calcular_el_valor_de_nlin_ncol(&arg1,NULL);
}

void MainWindow::on_spinBox_grid_nhregions_valueChanged(int arg1)
{
    calcular_el_valor_de_nlin_ncol(&arg1,NULL);
}

void MainWindow::on_spinBox_grid_c1_line_valueChanged(int arg1)
{
    calcular_el_valor_de_nlin_ncol(&arg1,NULL);
}

void MainWindow::on_spinBox_grid_c1_column_valueChanged(int arg1)
{
    calcular_el_valor_de_nlin_ncol(&arg1,NULL);
}

void MainWindow::on_spinBox_grid_c2_line_valueChanged(int arg1)
{
    calcular_el_valor_de_nlin_ncol(&arg1,NULL);
}

void MainWindow::on_spinBox_grid_c2_column_valueChanged(int arg1)
{
    calcular_el_valor_de_nlin_ncol(&arg1,NULL);
}

