
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QDir>

#include <QUrl>
#include <QDebug>
#include <QDesktopServices>

#include <QMessageBox>
#include <QFileDialog>

#include <string>       // std::string
#include <iostream>     // std::cout
#include <sstream> // std::stringstream


void MainWindow::on_actionTutorial_triggered()
{
    QString strtemp=QString(this->progdir)+QDir::separator()+
                            ".."+QDir::separator()+
                            "share"+QDir::separator()+
                            "doc"+QDir::separator()+
                            APP_TARGET+QDir::separator()+
                            APP_TARGET+".pdf";
#if defined(_WINDOWS_) || defined(_WIN32) || defined(WIN32)
    strtemp="file:"+strtemp;
#endif
    qDebug() <<"Opening:"<<strtemp;
    qDebug() << QDesktopServices::openUrl(QUrl(strtemp, QUrl::TolerantMode));
}


void MainWindow::on_actionAbout_triggered()
{
    std::stringstream about;

    about
     <<"<center><b> "<<APP_TARGET<<"</b></center><br>"
     <<"<b>version:</b> "<<APP_VERSION<<"<br>"
     <<"<b>license:</b> GPL<br>"
     <<"<b>homepage:</b> <a href='"<<APP_HOMEPAGE<<"'>"<<APP_HOMEPAGE<<"</a><br>"
     <<"<b>author:</b> Fernando Pujaico Rivera<br>"
     <<"<b>email:</b> fernando.pujaico.rivera@gmail.com<br>";

    QMessageBox::about(this,"About the program", QString::fromStdString(about.str()));
}

void MainWindow::on_actionLoad_regions_triggered()
{
    load_regions("");
}

void MainWindow::on_actionSave_regions_triggered()
{
    save_in_outfile();
}


void MainWindow::on_actionRemove_regions_triggered()
{
    ui->statusBar->showMessage(QString::number(this->pointl_p0.size())+tr(" regions removed"),3000);
    clear_regions();
    load_imagefile(ui->lineEdit_imagefile->text());
}

void MainWindow::on_actionScreenshot_triggered()
{
    QString  fileName;
    QString  origem;

    if(ui->lineEdit_outfile->text().count()!=0) // Save in the directory of outfile
        origem=QFileInfo(ui->lineEdit_outfile->text()).path()+QDir::separator()+"screenshot.bmp";
    else // Save in the home directory
        origem=QDir::homePath()+QDir::separator()+"screenshot.bmp";

    fileName = QFileDialog::getSaveFileName(this,
                                            tr("Define an output [PNG,BMP,JPG] filename"),
                                            origem,
                                            tr("Output image file (*.png *.bmp *.jpg)"));

    this->scene->clearSelection();                                                  // Selections would also render to the file
    this->scene->setSceneRect(scene->itemsBoundingRect());                          // Re-shrink the scene to it's bounding contents
    QImage image(scene->sceneRect().size().toSize(), QImage::Format_ARGB32);  // Create the image with the exact size of the shrunk scene
    image.fill(Qt::transparent);                                              // Start all pixels transparent

    QPainter painter(&image);
    this->scene->render(&painter);

    if(image.save(fileName,0,100)==true)
        ui->statusBar->showMessage(tr("Wrote the image file:")+origem,10000);
    else
        ui->statusBar->showMessage(tr("ERROR:: It was not wrote the image file:")+origem,10000);
}

void MainWindow::on_actionAbout_QT_libs_triggered()
{
    QMessageBox::aboutQt(this, "Qt :: a cross-platform application framework");
}
